@@ -9,6 +9,7 @@ import logging
 import os
 import shutil
 import subprocess
+import tempfile
 import textwrap
 from itertools import chain
 from pathlib import Path
@@ -60,6 +61,8 @@ BUCK_BUILDER_CACHE_PREFIX = ".buck_builder_cache"
 
 READER_WRITER_LOCK = "analysis_directory_reader_writer.lock"
 
+TEMPORARY_DIRECTORY_PREFIX = "pyre_tmp_"
+
 
 class NotWithinLocalConfigurationException(Exception):
     pass
@@ -131,16 +134,16 @@ class AnalysisDirectory:
 
     def process_updated_files(self, paths: List[str]) -> UpdatedPaths:
         """
-            Process a list of paths which were added/removed/updated, making any
-            necessary changes to the directory:
-                - For an AnalysisDirectory, nothing needs to be changed, since
-                  the mapping from source file to analysis file is 1:1.
-                - For a SharedAnalysisDirectory, the symbolic links (as well as
-                  the reverse-mapping we track) need to be updated to account for
-                  new and deleted files.
+        Process a list of paths which were added/removed/updated, making any
+        necessary changes to the directory:
+            - For an AnalysisDirectory, nothing needs to be changed, since
+              the mapping from source file to analysis file is 1:1.
+            - For a SharedAnalysisDirectory, the symbolic links (as well as
+              the reverse-mapping we track) need to be updated to account for
+              new and deleted files.
 
-            Return a list of files (corresponding to the given paths) that Pyre
-            should be tracking.
+        Return a list of files (corresponding to the given paths) that Pyre
+        should be tracking.
         """
         deleted_paths = [path for path in paths if not os.path.isfile(path)]
         tracked_paths = [
@@ -151,7 +154,7 @@ class AnalysisDirectory:
         tracked_paths.extend(deleted_paths)
         return UpdatedPaths(updated_paths=tracked_paths, deleted_paths=deleted_paths)
 
-    def cleanup(self) -> None:
+    def cleanup(self, delete_long_lasting_files: bool) -> None:
         pass
 
     @staticmethod
@@ -194,9 +197,10 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         isolate: bool = False,
         buck_builder: Optional[BuckBuilder] = None,
         configuration: Optional[Configuration] = None,
+        temporary_directories: Optional[List[str]] = None,
     ) -> None:
         self._source_directories: Set[str] = set(source_directories)
-        self._targets: Set[str] = set(targets)
+        self._targets: List[str] = targets
         self._original_directory = original_directory
         self._project_root = project_root
         self._filter_paths: Set[str] = filter_paths or set()
@@ -207,6 +211,7 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         )
         self._isolate = isolate
         self._buck_builder: BuckBuilder = buck_builder or buck.SimpleBuckBuilder()
+        self._temporary_directories: List[str] = temporary_directories or []
 
         # Mapping from source files in the project root to symbolic links in the
         # analysis directory.
@@ -263,8 +268,8 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         if not configuration or not configuration.logger:
             return
 
-        statistics.log(
-            configuration=self._configuration,
+        statistics.log_with_configuration(
+            configuration=configuration,
             category=statistics.LoggerCategory.BUCK_EVENTS,
             integers={
                 "runtime": int(runtime * 1000),
@@ -408,7 +413,10 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         )
         try:
             with SocketConnection(configuration.log_directory) as socket_connection:
-                socket_connection.perform_handshake(configuration.version_hash)
+                socket_connection.perform_handshake(
+                    configuration.get_version_hash_respecting_override()
+                    or "unversioned"
+                )
                 socket_connection.send(show_status_message)
         except (
             SocketException,
@@ -645,25 +653,33 @@ class SharedAnalysisDirectory(AnalysisDirectory):
         with self.acquire_writer_lock():
             return self._process_updated_files(paths)
 
-    def _directories_to_clean_up(self) -> List[str]:
-        if not self._isolate:
+    def _directories_to_clean_up(self, delete_long_lasting_files: bool) -> List[str]:
+        if not delete_long_lasting_files:
             return []
 
-        if not isinstance(self._buck_builder, buck.FastBuckBuilder):
+        if isinstance(self._buck_builder, buck.SimpleBuckBuilder):
             return [self.get_root()]
 
+        if isinstance(self._buck_builder, buck.SourceDatabaseBuckBuilder):
+            return [self.get_root(), *self._temporary_directories]
+
         project_name = _get_project_name(self._isolate, self._local_configuration_root)
         buck_builder_cache_path = os.path.join(
             self.get_scratch_directory(), f"{BUCK_BUILDER_CACHE_PREFIX}_{project_name}"
         )
-        return [self.get_root(), buck_builder_cache_path]
+        return [self.get_root(), buck_builder_cache_path, *self._temporary_directories]
 
-    def cleanup(self) -> None:
-        try:
-            for directory in self._directories_to_clean_up():
+    def cleanup(self, delete_long_lasting_files: bool) -> None:
+        delete_long_lasting_files = delete_long_lasting_files or self._isolate
+        directories_to_clean_up = self._directories_to_clean_up(
+            delete_long_lasting_files
+        )
+        LOG.debug(f"Cleaning up in the analysis directory: {directories_to_clean_up}.")
+        for directory in directories_to_clean_up:
+            try:
                 shutil.rmtree(directory)
-        except Exception as exception:
-            LOG.debug(f"Could not clean up analysis directory: {exception}")
+            except Exception as exception:
+                LOG.debug(f"Could not clean up analysis directory: {exception}")
 
     def _clear(self) -> None:
         root = self.get_root()
@@ -738,9 +754,9 @@ def _get_buck_builder(
     buck_mode: Optional[str],
     relative_local_root: Optional[str],
     isolate: bool,
-) -> BuckBuilder:
+) -> Tuple[BuckBuilder, List[str]]:
     if not configuration.use_buck_builder:
-        return buck.SimpleBuckBuilder()
+        return (buck.SimpleBuckBuilder(), [])
 
     buck_root = find_buck_root(project_root)
     if not buck_root:
@@ -748,17 +764,30 @@ def _get_buck_builder(
             f"No Buck configuration at `{project_root}` or any of its ancestors."
         )
 
+    output_directory = tempfile.mkdtemp(prefix=TEMPORARY_DIRECTORY_PREFIX)
+
     if configuration.use_buck_source_database:
-        return buck.SourceDatabaseBuckBuilder(buck_root=buck_root, buck_mode=buck_mode)
+        return (
+            buck.SourceDatabaseBuckBuilder(
+                buck_root=buck_root,
+                buck_mode=buck_mode,
+                output_directory=output_directory,
+            ),
+            [output_directory],
+        )
 
     project_name = _get_project_name(
         isolate_per_process=isolate, relative_local_root=relative_local_root
     )
-    return buck.FastBuckBuilder(
-        buck_root=buck_root,
-        buck_builder_binary=configuration.buck_builder_binary,
-        buck_mode=buck_mode,
-        project_name=project_name,
+    return (
+        buck.FastBuckBuilder(
+            buck_root=buck_root,
+            buck_builder_binary=configuration.buck_builder_binary,
+            buck_mode=buck_mode,
+            project_name=project_name,
+            output_directory=output_directory,
+        ),
+        [output_directory],
     )
 
 
@@ -783,8 +812,8 @@ def resolve_analysis_directory(
 
     # Only read from the configuration if no explicit targets are passed in.
     if not source_directories and not targets:
-        source_directories = configuration.source_directories
-        targets = configuration.targets
+        source_directories = list(configuration.source_directories)
+        targets = list(configuration.targets)
     else:
         source_directories = source_directories or []
         targets = targets or []
@@ -816,7 +845,7 @@ def resolve_analysis_directory(
             ],
         )
     else:
-        buck_builder = _get_buck_builder(
+        buck_builder, temporary_directories = _get_buck_builder(
             project_root, configuration, buck_mode, relative_local_root, isolate
         )
 
@@ -828,12 +857,13 @@ def resolve_analysis_directory(
             project_root=project_root,
             filter_paths=filter_paths,
             local_configuration_root=local_configuration_root,
-            extensions=configuration.extensions,
+            extensions=configuration.get_valid_extensions(),
             search_path=[
                 search_path.path()
                 for search_path in configuration.get_existent_search_paths()
             ],
             isolate=isolate,
             configuration=configuration,
+            temporary_directories=temporary_directories,
         )
     return analysis_directory
