@@ -3,7 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-import argparse
 import logging
 from logging import Logger
 from typing import List, Optional
@@ -33,30 +32,6 @@ class Check(Reporting):
             command_arguments, original_directory, configuration, analysis_directory
         )
 
-    @staticmethod
-    def from_arguments(
-        arguments: argparse.Namespace,
-        original_directory: str,
-        configuration: Configuration,
-        analysis_directory: Optional[AnalysisDirectory] = None,
-    ) -> "Check":
-        return Check(
-            command_arguments.CommandArguments.from_arguments(arguments),
-            original_directory,
-            configuration=configuration,
-            analysis_directory=analysis_directory,
-        )
-
-    @classmethod
-    def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
-        check = parser.add_parser(
-            cls.NAME,
-            epilog="""
-          Runs a one-time check of a project without initializing a type check server.
-        """,
-        )
-        check.set_defaults(command=cls.from_arguments)
-
     def generate_analysis_directory(self) -> AnalysisDirectory:
         return resolve_analysis_directory(
             self._command_arguments.source_directories,
@@ -77,11 +52,13 @@ class Check(Reporting):
         )
         if len(filter_directories):
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
-        flags.extend(["-workers", str(self._configuration.number_of_workers)])
+        flags.extend(["-workers", str(self._configuration.get_number_of_workers())])
+
+        typeshed = self._configuration.get_typeshed_respecting_override()
         search_path = [
             search_path.command_line_argument()
             for search_path in self._configuration.get_existent_search_paths()
-        ] + typeshed_search_path(self._configuration.typeshed)
+        ] + (typeshed_search_path(typeshed) if typeshed is not None else [])
 
         ignore_all_errors_paths = (
             self._configuration.get_existent_ignore_all_errors_paths()
@@ -90,12 +67,13 @@ class Check(Reporting):
             flags.extend(
                 ["-ignore-all-errors", ";".join(sorted(ignore_all_errors_paths))]
             )
+        flags.extend(["-source-path", self._analysis_directory.get_root()])
         if search_path:
             flags.extend(["-search-path", ",".join(search_path)])
         excludes = self._configuration.excludes
         for exclude in excludes:
             flags.extend(["-exclude", exclude])
-        extensions = self._configuration.extensions
+        extensions = self._configuration.get_valid_extensions()
         for extension in extensions:
             flags.extend(["-extension", extension])
         return flags
