@@ -19,18 +19,14 @@ from typing import IO, Iterable, List, Optional
 
 from .. import command_arguments, json_rpc, log, recently_used_configurations, terminal
 from ..analysis_directory import AnalysisDirectory, resolve_analysis_directory
-from ..configuration import Configuration
+from ..configuration import Configuration, InvalidConfiguration
 from ..exceptions import EnvironmentException
-from ..filesystem import readable_directory, remove_if_exists
+from ..filesystem import remove_if_exists
 from ..log import StreamLogger
 from ..process import Process
 from ..socket_connection import SocketConnection, SocketException
 
 
-TEXT: str = "text"
-JSON: str = "json"
-
-
 LOG: logging.Logger = logging.getLogger(__name__)
 
 
@@ -69,7 +65,9 @@ class ProfileOutput(enum.Enum):
     INCREMENTAL_UPDATES: str = "incremental_updates"
     INDIVIDUAL_TABLE_SIZES: str = "individual_table_sizes"
     TOTAL_SHARED_MEMORY_SIZE_OVER_TIME: str = "total_shared_memory_size_over_time"
-    TOTAL_SHARED_MEMORY_SIZE_OVER_TIME_GRAPH: str = "total_shared_memory_size_over_time_graph"  # noqa B950
+    TOTAL_SHARED_MEMORY_SIZE_OVER_TIME_GRAPH: str = (
+        "total_shared_memory_size_over_time_graph"  # noqa B950
+    )
 
     def __str__(self) -> str:
         return self.value
@@ -145,184 +143,6 @@ class CommandParser(ABC):
     def __init__(self) -> None:
         pass
 
-    @classmethod
-    def add_arguments(cls, parser: argparse.ArgumentParser) -> None:
-        parser.add_argument(
-            "-l", "--local-configuration", type=str, help="Use a local configuration"
-        )
-
-        parser.add_argument(
-            "--version",
-            action="store_true",
-            help="Print the client and binary versions of Pyre.",
-        )
-
-        parser.add_argument("--debug", action="store_true", help=argparse.SUPPRESS)
-        parser.add_argument("--sequential", action="store_true", help=argparse.SUPPRESS)
-        parser.add_argument("--strict", action="store_true", help=argparse.SUPPRESS)
-        parser.add_argument(
-            "--additional-check", action="append", help=argparse.SUPPRESS
-        )
-
-        parser.add_argument(
-            "--show-error-traces",
-            action="store_true",
-            help="Display errors trace information",
-        )
-
-        # Logging.
-        parser.add_argument(
-            "--output", choices=[TEXT, JSON], default=TEXT, help="How to format output"
-        )
-        parser.add_argument(
-            "--enable-profiling", action="store_true", help=argparse.SUPPRESS
-        )
-        parser.add_argument(
-            "--enable-memory-profiling", action="store_true", help=argparse.SUPPRESS
-        )
-        parser.add_argument(
-            "-n",
-            "--noninteractive",
-            action="store_true",
-            help="Disable interactive logging",
-        )
-        parser.add_argument(
-            "--logging-sections", help=argparse.SUPPRESS  # Enable sectional logging.
-        )
-        parser.add_argument(
-            "--log-identifier",
-            default="",
-            help=argparse.SUPPRESS,  # Add given identifier to logged samples.
-        )
-
-        # Directory where Pyre places its log files and artifacts.
-        parser.add_argument("--dot-pyre-directory", type=Path, help=argparse.SUPPRESS)
-        parser.add_argument(
-            "--logger", help=argparse.SUPPRESS  # Specify custom logging binary.
-        )
-        parser.add_argument("--formatter", help=argparse.SUPPRESS)
-
-        # Link tree determination.
-        buck_arguments = parser.add_argument_group("buck")
-        buck_arguments.add_argument(
-            "--target",
-            action="append",
-            dest="targets",
-            default=[],
-            help="The buck target to check",
-        )
-
-        use_buck_builder_group = buck_arguments.add_mutually_exclusive_group()
-        use_buck_builder_group.add_argument(
-            "--use-buck-builder",
-            action="store_const",
-            const=True,
-            help="Use Pyre's experimental builder for Buck projects.",
-        )
-        use_buck_builder_group.add_argument(
-            "--use-legacy-buck-builder",
-            dest="use_buck_builder",
-            action="store_const",
-            const=False,
-            help="Use the legacy builder for Buck projects.",
-        )
-
-        buck_arguments.add_argument(
-            "--buck-mode", type=str, help="Mode to pass to `buck query`"
-        )
-        buck_arguments.add_argument(
-            "--use-buck-source-database",
-            action="store_const",
-            const=True,
-            help=argparse.SUPPRESS,
-        )
-
-        source_directories = parser.add_argument_group("source-directories")
-        source_directories.add_argument(
-            "--source-directory",
-            action="append",
-            dest="source_directories",
-            default=[],
-            help="The source directory to check",
-            type=os.path.abspath,
-        )
-        source_directories.add_argument(
-            "--filter-directory", help=argparse.SUPPRESS  # override filter directory
-        )
-        parser.add_argument(
-            "--no-saved-state",
-            action="store_true",
-            help="Don't attempt to load Pyre from a saved state.",
-        )
-
-        # Handling of search path
-        parser.add_argument(
-            "--search-path",
-            action="append",
-            default=[],
-            type=readable_directory,
-            help="Add an additional directory of modules and stubs to include"
-            " in the type environment",
-        )
-
-        parser.add_argument(
-            "--binary",
-            default=None,
-            type=executable_file,
-            help="Location of the pyre binary",
-        )
-
-        parser.add_argument(
-            "--buck-builder-binary",
-            default=None,
-            help="Location of the buck builder binary",
-        )
-
-        parser.add_argument(
-            "--exclude",
-            action="append",
-            default=[],
-            help="Exclude files and directories matching this regexp from parsing",
-        )
-
-        # Typeshed stubs location
-        parser.add_argument(
-            "--typeshed",
-            default=None,
-            type=readable_directory,
-            help="Location of the typeshed stubs",
-        )
-        parser.add_argument(
-            "--save-initial-state-to",
-            default=None,
-            help="Path to serialize pyre's initial state to.",
-        )
-        parser.add_argument(
-            "--load-initial-state-from", default=None, type=str, help=argparse.SUPPRESS
-        )
-        parser.add_argument(
-            "--changed-files-path", default=None, type=str, help=argparse.SUPPRESS
-        )
-        parser.add_argument(
-            "--saved-state-project", default=None, type=str, help=argparse.SUPPRESS
-        )
-        parser.add_argument(
-            "--features", default=None, type=str, help=argparse.SUPPRESS
-        )
-        # Temporary flag to help migrate to json sockets for incremental and query
-        # commands.
-        parser.add_argument(
-            "--use-json-sockets",
-            action="store_true",
-            default=False,
-            help=argparse.SUPPRESS,
-        )
-
-    @classmethod
-    @abstractmethod
-    def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
-        pass
-
     @abstractmethod
     def _run(self) -> None:
         """ Abstract method expected to be overridden by subclasses. """
@@ -366,8 +186,10 @@ class Command(CommandParser, ABC):
             self._noninteractive = True
 
         self._configuration: Configuration = configuration
-        self._version_hash: str = self._configuration.version_hash
-        self._taint_models_path: List[str] = self._configuration.taint_models_path
+        self._version_hash: str = (
+            self._configuration.get_version_hash_respecting_override() or "unversioned"
+        )
+        self._taint_models_path: List[str] = list(self._configuration.taint_models_path)
 
         self._analysis_directory: AnalysisDirectory = (
             analysis_directory or self.generate_analysis_directory()
@@ -411,7 +233,7 @@ class Command(CommandParser, ABC):
         pass
 
     def cleanup(self) -> None:
-        self._analysis_directory.cleanup()
+        self._analysis_directory.cleanup(delete_long_lasting_files=False)
 
     def _flags(self) -> List[str]:
         flags = []
@@ -486,7 +308,10 @@ class Command(CommandParser, ABC):
                 "`{}` is not a link tree.".format(self._analysis_directory.get_root())
             )
 
-        client_command = [self._configuration.binary, command]
+        binary = self._configuration.get_binary_respecting_override()
+        if binary is None:
+            raise InvalidConfiguration("Cannot find any pyre binary.")
+        client_command = [binary, command]
         client_command.extend(self._flags())
         client_command.append(self._analysis_directory.get_root())
 
