@@ -5,8 +5,8 @@
 
 # pyre-unsafe
 
-import argparse
 import functools
+import json
 import logging
 import os
 import re
@@ -16,7 +16,7 @@ import sys
 from collections import defaultdict
 from logging import Logger
 from pathlib import Path
-from typing import IO, Any, List, Optional, Sequence, Set, Union
+from typing import IO, Any, Dict, List, Optional, Sequence, Set, Union
 
 import libcst
 from libcst.codemod import CodemodContext
@@ -25,10 +25,12 @@ from typing_extensions import Final
 
 from .. import command_arguments, log
 from ..analysis_directory import AnalysisDirectory
+from ..annotation_collector import AnnotationCollector
 from ..configuration import Configuration
 from ..error import Error
-from .command import JSON, Command, Result, typeshed_search_path
+from .command import Command, Result, typeshed_search_path
 from .reporting import Reporting
+from .statistics import _get_paths, _parse_paths, parse_path_to_module
 
 
 LOG: Logger = logging.getLogger(__name__)
@@ -76,7 +78,7 @@ class FunctionStub:
         return " -> " + dequalify(self.actual) if self.actual else ""
 
     def _get_parameter_string(self) -> str:
-        """ Depending on if an argument has a type, the style for default values
+        """Depending on if an argument has a type, the style for default values
         changes. E.g.
            def fun(x=5)
            def fun(x : int = 5)
@@ -236,9 +238,9 @@ class StubFile:
 
     def to_string(self) -> str:
         """We currently ignore nested classes, i.e.:
-          class X:
-              class Y:
-                  [ALL OF THIS IS IGNORED]
+        class X:
+            class Y:
+                [ALL OF THIS IS IGNORED]
         """
         classes = defaultdict(list)
         typing_imports = set()
@@ -418,10 +420,31 @@ def annotate_from_existing_stubs(
         subprocess.call(formatter, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
 
 
-def file_exists(path):
-    if not os.path.exists(path):
-        raise argparse.ArgumentTypeError("ERROR: " + str(path) + " does not exist")
-    return path
+def _existing_annotations_as_errors(
+    modules: Dict[Path, Optional[libcst.Module]], project_root: str
+) -> List[Error]:
+    errors = []
+    for path_name, module in modules.items():
+        path_name = str(path_name)
+        path = path_name.replace(".py", "").replace(project_root + "/", "")
+        collector = AnnotationCollector(path)
+        if module is not None:
+            module.visit(collector)
+        for stub in collector.stubs:
+            errors.append(
+                Error(
+                    error={
+                        "path": path_name.replace(project_root + "/", ""),
+                        "inference": stub,
+                        "line": 0,
+                        "column": 0,
+                        "code": 0,
+                        "name": "example_string",
+                        "description": "example description",
+                    }
+                )
+            )
+    return errors
 
 
 class Infer(Reporting):
@@ -429,7 +452,7 @@ class Infer(Reporting):
 
     def __init__(
         self,
-        command_arguments: command_arguments.CommandArguments,
+        arguments: command_arguments.CommandArguments,
         original_directory: str,
         *,
         configuration: Configuration,
@@ -441,9 +464,10 @@ class Infer(Reporting):
         errors_from_stdin: bool,
         annotate_from_existing_stubs: bool,
         debug_infer: bool,
+        full_stub_paths: Optional[List[str]],
     ) -> None:
         super(Infer, self).__init__(
-            command_arguments, original_directory, configuration, analysis_directory
+            arguments, original_directory, configuration, analysis_directory
         )
         self._print_errors = print_errors
         self._full_only = full_only
@@ -452,89 +476,18 @@ class Infer(Reporting):
         self._errors_from_stdin = errors_from_stdin
         self._annotate_from_existing_stubs = annotate_from_existing_stubs
         self._debug_infer = debug_infer
-        self._ignore_infer: List[str] = self._configuration.ignore_infer
+        self._ignore_infer: List[
+            str
+        ] = self._configuration.get_existent_ignore_infer_paths()
+        self._full_stub_paths: Final[Optional[List[str]]] = full_stub_paths
 
         self._show_error_traces = True
-        self._output = JSON
-
-    @staticmethod
-    def from_arguments(
-        arguments: argparse.Namespace,
-        original_directory: str,
-        configuration: Configuration,
-        analysis_directory: Optional[AnalysisDirectory] = None,
-    ) -> "Infer":
-        return Infer(
-            command_arguments.CommandArguments.from_arguments(arguments),
-            original_directory,
-            configuration=configuration,
-            analysis_directory=analysis_directory,
-            print_errors=arguments.print_only,
-            full_only=arguments.full_only,
-            recursive=arguments.recursive,
-            in_place=arguments.in_place,
-            errors_from_stdin=arguments.errors_from_stdin,
-            annotate_from_existing_stubs=arguments.annotate_from_existing_stubs,
-            debug_infer=arguments.debug_infer,
-        )
-
-    @classmethod
-    def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
-        infer = parser.add_parser(cls.NAME)
-        infer.set_defaults(command=cls.from_arguments)
-        infer.add_argument(
-            "-p",
-            "--print-only",
-            action="store_true",
-            help="Print raw JSON errors to standard output, "
-            + "without converting to stubs or annnotating.",
-        )
-        infer.add_argument(
-            "-f",
-            "--full-only",
-            action="store_true",
-            help="Only output fully annotated functions. Requires infer flag.",
-        )
-        infer.add_argument(
-            "-r",
-            "--recursive",
-            action="store_true",
-            help="Recursively run infer until no new annotations are generated."
-            + " Requires infer flag.",
-        )
-        infer.add_argument(
-            "-i",
-            "--in-place",
-            nargs="*",
-            metavar="path",
-            type=file_exists,
-            help="Add annotations to functions in selected paths."
-            + " Takes a set of files and folders to add annotations to."
-            + " If no paths are given, all functions are annotated."
-            + " WARNING: Modifies original files and requires infer flag and retype",
-        )
-        infer.add_argument(
-            "--json",
-            action="store_true",
-            dest="errors_from_stdin",
-            help="Accept JSON input instead of running full check.",
-        )
-        infer.add_argument(
-            "--annotate-from-existing-stubs",
-            action="store_true",
-            help="Add annotations from existing stubs.",
-        )
-        infer.add_argument(
-            "--debug-infer",
-            action="store_true",
-            help="Print error message when file fails to annotate.",
-        )
+        self._output = command_arguments.JSON
 
     def run(self) -> Command:
-        self._analysis_directory.prepare()
         if self._annotate_from_existing_stubs:
             if self._in_place is None:
-                raise argparse.ArgumentTypeError(
+                raise ValueError(
                     "--annotate-from-existing-stubs cannot be used without the \
                     --in-place argument"
                 )
@@ -550,10 +503,26 @@ class Infer(Reporting):
                 self._debug_infer,
             )
             return self
+        if self._full_stub_paths is not None:
+            if self._full_stub_paths:
+                paths = _parse_paths([Path(path) for path in self._full_stub_paths])
+            else:
+                paths = _get_paths(Path(self._original_directory))
+            modules = {path: parse_path_to_module(path) for path in paths}
+            errors = _existing_annotations_as_errors(
+                modules, self._configuration.project_root
+            )
+            type_directory = Path(
+                os.path.join(self._configuration.log_directory, "types")
+            )
+            stubs = generate_stub_files(full_only=False, errors=errors)
+            write_stubs_to_disk(stubs, type_directory)
+            return self
         if self._errors_from_stdin:
             result = self._get_errors_from_stdin()
             from_stdin = True
         else:
+            self._analysis_directory.prepare()
             result = self._call_client(command=Infer.NAME)
             from_stdin = False
         errors = self._get_errors(result, bypass_filtering=True, from_stdin=from_stdin)
@@ -586,12 +555,16 @@ class Infer(Reporting):
         )
         if len(filter_directories):
             flags.extend(["-filter-directories", ";".join(sorted(filter_directories))])
+
+        typeshed = self._configuration.get_typeshed_respecting_override()
         search_path = [
             search_path.command_line_argument()
             for search_path in self._configuration.get_existent_search_paths()
-        ] + typeshed_search_path(self._configuration.typeshed)
+        ] + (typeshed_search_path(typeshed) if typeshed is not None else [])
+        flags.extend(["-source-path", self._analysis_directory.get_root()])
         if search_path:
             flags.extend(["-search-path", ",".join(search_path)])
+
         if len(self._ignore_infer) > 0:
             flags.extend(["-ignore-infer", ";".join(self._ignore_infer)])
         return flags
@@ -610,4 +583,4 @@ class Infer(Reporting):
 
     def _get_errors_from_stdin(self) -> Result:
         input = sys.stdin.read()
-        return Result(0, input)
+        return Result(0, json.dumps({"errors": json.loads(input)}))
