@@ -3,7 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-import argparse
 from typing import Optional
 
 from .. import command_arguments
@@ -38,51 +37,6 @@ class Restart(Command):
         self._use_watchman: bool = use_watchman
         self._incremental_style: IncrementalStyle = incremental_style
 
-    @staticmethod
-    def from_arguments(
-        arguments: argparse.Namespace,
-        original_directory: str,
-        configuration: Configuration,
-        analysis_directory: Optional[AnalysisDirectory] = None,
-    ) -> "Restart":
-        return Restart(
-            command_arguments.CommandArguments.from_arguments(arguments),
-            original_directory,
-            configuration=configuration,
-            analysis_directory=analysis_directory,
-            terminal=arguments.terminal,
-            store_type_check_resolution=arguments.store_type_check_resolution,
-            use_watchman=not arguments.no_watchman,
-            incremental_style=arguments.incremental_style,
-        )
-
-    @classmethod
-    def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
-        restart = parser.add_parser(
-            cls.NAME, epilog="Restarts a server. Equivalent to `pyre stop && pyre`."
-        )
-        restart.set_defaults(command=cls.from_arguments)
-        restart.add_argument(
-            "--terminal", action="store_true", help="Run the server in the terminal."
-        )
-        restart.add_argument(
-            "--store-type-check-resolution",
-            action="store_true",
-            help="Store extra information for `types` queries.",
-        )
-        restart.add_argument(
-            "--no-watchman",
-            action="store_true",
-            help="Do not spawn a watchman client in the background.",
-        )
-        restart.add_argument(
-            "--incremental-style",
-            type=IncrementalStyle,
-            choices=list(IncrementalStyle),
-            default=IncrementalStyle.FINE_GRAINED,
-            help="How to approach doing incremental checks.",
-        )
-
     def generate_analysis_directory(self) -> AnalysisDirectory:
         return resolve_analysis_directory(
             self._command_arguments.source_directories,
