@@ -3,7 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-import argparse
 import errno
 import logging
 import os
@@ -50,49 +49,6 @@ class Start(Reporting):
 
         self._enable_logging_section("environment")
 
-    @staticmethod
-    def from_arguments(
-        arguments: argparse.Namespace,
-        original_directory: str,
-        configuration: Configuration,
-        analysis_directory: Optional[AnalysisDirectory] = None,
-    ) -> "Start":
-        return Start(
-            command_arguments.CommandArguments.from_arguments(arguments),
-            original_directory,
-            configuration=configuration,
-            analysis_directory=analysis_directory,
-            terminal=arguments.terminal,
-            store_type_check_resolution=arguments.store_type_check_resolution,
-            use_watchman=not arguments.no_watchman,
-            incremental_style=arguments.incremental_style,
-        )
-
-    @classmethod
-    def add_subparser(cls, parser: argparse._SubParsersAction) -> None:
-        start = parser.add_parser(cls.NAME, epilog="Starts a pyre server as a daemon.")
-        start.set_defaults(command=cls.from_arguments)
-        start.add_argument(
-            "--terminal", action="store_true", help="Run the server in the terminal."
-        )
-        start.add_argument(
-            "--store-type-check-resolution",
-            action="store_true",
-            help="Store extra information for `types` queries.",
-        )
-        start.add_argument(
-            "--no-watchman",
-            action="store_true",
-            help="Do not spawn a watchman client in the background.",
-        )
-        start.add_argument(
-            "--incremental-style",
-            type=IncrementalStyle,
-            choices=list(IncrementalStyle),
-            default=IncrementalStyle.FINE_GRAINED,
-            help="How to approach doing incremental checks.",
-        )
-
     def _start_configuration_monitor(self) -> None:
         if self._use_watchman:
             configuration_monitor.ConfigurationMonitor(
@@ -102,61 +58,47 @@ class Start(Reporting):
                 self._configuration.project_root,
                 self._original_directory,
                 self._configuration.local_root,
-                self._configuration.other_critical_files,
+                list(self._configuration.other_critical_files),
             ).daemonize()
 
     def _run(self) -> None:
-        blocking = False
         lock = os.path.join(self._configuration.log_directory, "client.lock")
-        while True:
-            # Be optimistic in grabbing the lock in order to provide users with
-            # a message when the lock is being waited on.
+        LOG.info("Waiting on the pyre client lock.")
+        with filesystem.acquire_lock(lock, blocking=True):
+            self._start_configuration_monitor()
+            # This unsafe call is OK due to the client lock always
+            # being acquired before starting a server - no server can
+            # spawn in the interim which would cause a race.
             try:
-                with filesystem.acquire_lock(lock, blocking):
-                    self._start_configuration_monitor()
-                    # This unsafe call is OK due to the client lock always
-                    # being acquired before starting a server - no server can
-                    # spawn in the interim which would cause a race.
-                    try:
-                        with filesystem.acquire_lock(
-                            os.path.join(
-                                self._configuration.log_directory,
-                                "server",
-                                "server.lock",
-                            ),
-                            blocking=False,
-                        ):
-                            pass
-                    except OSError:
-                        LOG.warning(
-                            "Server at `%s` exists, skipping.",
-                            self._analysis_directory.get_root(),
-                        )
-                        return
+                with filesystem.acquire_lock(
+                    os.path.join(
+                        self._configuration.log_directory, "server", "server.lock"
+                    ),
+                    blocking=False,
+                ):
+                    pass
+            except OSError:
+                LOG.warning(
+                    "Server at `%s` exists, skipping.",
+                    self._analysis_directory.get_root(),
+                )
+                return
 
-                    self._analysis_directory.prepare()
+            self._analysis_directory.prepare()
 
-                    self._call_client(command=self.NAME).check()
+            self._call_client(command=self.NAME).check()
 
-                    if self._use_watchman:
-                        try:
-                            file_monitor = project_files_monitor.ProjectFilesMonitor(
-                                self._configuration,
-                                self._configuration.project_root,
-                                self._analysis_directory,
-                            )
-                            file_monitor.daemonize()
-                            LOG.debug("Initialized file monitor.")
-                        except project_files_monitor.MonitorException as error:
-                            LOG.warning("Failed to initialize file monitor: %s", error)
-
-                    return
-            except OSError as exception:
-                if exception.errno == errno.EAGAIN:
-                    blocking = True
-                    LOG.info("Waiting on the pyre client lock.")
-                else:
-                    raise exception
+            if self._use_watchman:
+                try:
+                    file_monitor = project_files_monitor.ProjectFilesMonitor(
+                        self._configuration,
+                        self._configuration.project_root,
+                        self._analysis_directory,
+                    )
+                    file_monitor.daemonize()
+                    LOG.debug("Initialized file monitor.")
+                except project_files_monitor.MonitorException as error:
+                    LOG.warning("Failed to initialize file monitor: %s", error)
 
     def _flags(self) -> List[str]:
         flags = super()._flags()
@@ -211,15 +153,19 @@ class Start(Reporting):
         flags.extend(
             [
                 "-workers",
-                str(self._configuration.number_of_workers),
+                str(self._configuration.get_number_of_workers()),
                 "-expected-binary-version",
-                self._configuration.version_hash,
+                self._configuration.get_version_hash_respecting_override()
+                or "unversioned",
             ]
         )
+
+        typeshed = self._configuration.get_typeshed_respecting_override()
         search_path = [
             search_path.command_line_argument()
             for search_path in self._configuration.get_existent_search_paths()
-        ] + typeshed_search_path(self._configuration.typeshed)
+        ] + (typeshed_search_path(typeshed) if typeshed is not None else [])
+        flags.extend(["-source-path", self._analysis_directory.get_root()])
         if search_path:
             flags.extend(["-search-path", ",".join(search_path)])
 
@@ -227,7 +173,7 @@ class Start(Reporting):
         for exclude in excludes:
             flags.extend(["-exclude", exclude])
 
-        extensions = self._configuration.extensions
+        extensions = self._configuration.get_valid_extensions()
         for extension in extensions:
             flags.extend(["-extension", extension])
 
