@@ -37,10 +37,10 @@ def _log_paths(message: str, paths: Sequence[str]) -> None:
 
 class ProjectFilesMonitor(Subscriber):
     """
-        Logs from this monitor are found in
-        .pyre/<local root>/file_monitor/file_monitor.log
-        One file monitor is spawned per pyre server. When a server is stopped,
-        the process of pyre file monitor associated to it is killed.
+    Logs from this monitor are found in
+    .pyre/<local root>/file_monitor/file_monitor.log
+    One file monitor is spawned per pyre server. When a server is stopped,
+    the process of pyre file monitor associated to it is killed.
     """
 
     NAME = "file_monitor"
@@ -51,19 +51,23 @@ class ProjectFilesMonitor(Subscriber):
         project_root: str,
         analysis_directory: AnalysisDirectory,
     ) -> None:
-        super(ProjectFilesMonitor, self).__init__(self.base_path(configuration))
-        self._configuration = configuration
+        super(ProjectFilesMonitor, self).__init__(
+            self.base_path(configuration), configuration
+        )
         self._analysis_directory = analysis_directory
 
         self._extensions: Set[str] = set(
-            ["py", "pyi", "thrift"] + configuration.extensions
+            ["py", "pyi", "thrift"]
+            + [extension[1:] for extension in configuration.get_valid_extensions()]
         )
 
         self._watchman_path: str = self._find_watchman_path(project_root)
 
         self.socket_connection = SocketConnection(self._configuration.log_directory)
         self.socket_connection.connect()
-        self.socket_connection.perform_handshake(self._configuration.version_hash)
+        self.socket_connection.perform_handshake(
+            self._configuration.get_version_hash_respecting_override() or "unversioned"
+        )
 
     @property
     def _name(self) -> str:
@@ -154,4 +158,11 @@ class ProjectFilesMonitor(Subscriber):
         pid_path = watchman.compute_pid_path(
             ProjectFilesMonitor.base_path(configuration), ProjectFilesMonitor.NAME
         )
-        return Process.is_alive(Path(pid_path))
+        is_alive = Process.is_alive(Path(pid_path))
+        if not is_alive:
+            LOG.debug("The file monitor is down.")
+        return is_alive
+
+    def cleanup(self) -> None:
+        LOG.info("Cleaning up the analysis directory.")
+        self._analysis_directory.cleanup(delete_long_lasting_files=True)
