@@ -4,6 +4,7 @@
 # LICENSE file in the root directory of this source tree.
 
 import builtins
+import enum
 import itertools
 import os
 import sys
@@ -14,7 +15,6 @@ from typing import (
     Dict,
     Iterable,
     List,
-    NamedTuple,
     Optional,
     Set,
     Tuple,
@@ -24,7 +24,6 @@ from typing import (
 )
 
 import click
-import graphene
 import IPython
 from IPython import paths
 from IPython.core import page
@@ -49,20 +48,17 @@ from ..models import (
     Issue,
     IssueInstance,
     IssueInstanceSharedTextAssoc,
-    IssueInstanceTraceFrameAssoc,
     Run,
     RunStatus,
     SharedText,
     SharedTextKind,
-    SourceLocation,
     TraceFrame,
-    TraceFrameLeafAssoc,
     TraceKind,
     create as create_models,
 )
 from ..pipeline.base_parser import BaseParser
 from . import trace
-from .issues import Query
+from .issues import IssueQueryResult, Query
 from .trace import TraceFrameQueryResult, TraceTuple
 
 
@@ -80,41 +76,15 @@ CalleeText = aliased(SharedText)
 MessageText = aliased(SharedText)
 
 
-class IssueQueryResult(NamedTuple):
-    id: DBID
-    filename: str
-    location: SourceLocation
-    code: int
-    callable: str
-    message: str
-    min_trace_length_to_sources: int
-    min_trace_length_to_sinks: int
-
-
-class IssueQueryResultType(graphene.ObjectType):
-    class Meta:
-        interfaces = (graphene.relay.Node,)
-
-    issue_id = graphene.ID()
-    filename = graphene.String()
-    location = graphene.String()
-    code = graphene.Int()
-    callable = graphene.String()
-    message = graphene.String()
-    min_trace_length_to_sources = graphene.Int()
-    min_trace_length_to_sinks = graphene.Int()
-
-    # pyre-fixme[3]: Return type must be annotated.
-    # pyre-fixme[2]: Parameter must be annotated.
-    def resolve_issue_id(self, info):
-        # pyre-fixme[16]: `IssueQueryResultType` has no attribute `id`.
-        return self.id
-
-
 class ListFilterException(Exception):
     pass
 
 
+class LeafOrderBy(str, enum.Enum):
+    name = "name"
+    number_issues = "number_issues"
+
+
 class Interactive:
     help_message = f"""
 Commands =======================================================================
@@ -128,6 +98,7 @@ state                show the internal state of the tool for debugging
 runs                 list all completed static analysis runs
 issues               list all issues for the selected run
 frames               show trace frames independently of an issue
+leaves               list all leaves of issues for the selected run
 show                 show info about selected issue or trace frame
 
 == Selection commands ==
@@ -187,6 +158,7 @@ details              show additional information about the current trace frame
             "list": self.list_source_code,
             "frames": self.frames,
             "frame": self.frame,
+            "leaves": self.leaves,
             "parents": self.parents,
             "details": self.details,
             "analysis_output": self.analysis_output,
@@ -457,8 +429,7 @@ details              show additional information about the current trace frame
     @catch_user_error()
     # pyre-fixme[3]: Return type must be annotated.
     def show(self):
-        """ More details about the selected issue or trace frame.
-        """
+        """More details about the selected issue or trace frame."""
         self._verify_entrypoint_selected()
 
         if self.current_issue_instance_id != -1:
@@ -542,7 +513,7 @@ details              show additional information about the current trace frame
                     raise UserError("'filenames' should be str or list of str.")
                 if isinstance(filenames, str):
                     filenames = [filenames]
-                builder = builder.where_file_names_is_any_of(filenames)
+                builder = builder.where_path_is_any_of(filenames)
 
             if (exact_trace_length_to_sources is not None) and (
                 max_trace_length_to_sources is not None
@@ -620,9 +591,9 @@ details              show additional information about the current trace frame
                 builder = builder.where_exclude_features(exclude_features)
 
             issues = builder.get()
-            sources_list = builder.sources(issues)
-            sinks_list = builder.sinks(issues)
-            features_list = builder.features(issues)
+            sources_list = [builder.sources(issue.id) for issue in issues]
+            sinks_list = [builder.sinks(issue.id) for issue in issues]
+            features_list = [builder.features(issue.id) for issue in issues]
 
         issue_strings = []
         for issue, sources, sinks, features in zip(
@@ -778,6 +749,77 @@ details              show additional information about the current trace frame
         self._generate_trace_from_frame()
         self.show()
 
+    @catch_keyboard_interrupt()
+    @catch_user_error()
+    def leaves(
+        self,
+        kind: str = "sink",
+        use_pager: Optional[bool] = None,
+        limit: Optional[int] = None,
+        order_by: Optional[LeafOrderBy] = None,
+    ) -> None:
+        """Lists leaves of issues for the selected run.
+
+        Parameters (all optional):
+            kind: source|sink|feature (default: sink)       the type of leaves to show
+            use_pager: bool                                 use a unix style pager for output
+            limit: int (default: all)                       how many leaves to display
+            order_by: name|number_issues (default: random)  sort by a criteria
+        """
+        pager = self._resolve_pager(use_pager)
+        leaves: DefaultDict[str, int] = defaultdict(int)
+
+        text_kind = SharedTextKind.from_string(kind)
+        if text_kind is None:
+            raise UserError("Invalid kind.")
+
+        # Show leaf names instead of leaf kinds.
+        if text_kind == SharedTextKind.source:
+            text_kind = SharedTextKind.source_detail
+        elif text_kind == SharedTextKind.sink:
+            text_kind = SharedTextKind.sink_detail
+
+        with self.db.make_session() as session:
+            query = (
+                session.query(
+                    IssueInstanceSharedTextAssoc.shared_text_id, SharedText.contents
+                )
+                .join(
+                    SharedText,
+                    IssueInstanceSharedTextAssoc.shared_text_id == SharedText.id,
+                )
+                .join(
+                    IssueInstance,
+                    IssueInstanceSharedTextAssoc.issue_instance_id == IssueInstance.id,
+                )
+                .filter(IssueInstance.run_id == self.current_run_id)
+                .filter(SharedText.kind == text_kind)
+            )
+            for (_, name) in query:
+                leaves[name] += 1
+
+        query: Iterable[Tuple[str, int]]
+        if order_by == LeafOrderBy.name:
+            query = sorted(leaves.items(), key=lambda leaf: leaf[0])
+        elif order_by == LeafOrderBy.number_issues:
+            query = sorted(leaves.items(), key=lambda leaf: leaf[1], reverse=True)
+        elif order_by is None:
+            query = leaves.items()
+        else:
+            raise UserError("Invalid order_by method.")
+
+        if limit is not None:
+            query = itertools.islice(query, limit)
+
+        leaves_strings = []
+        for name, number_issues in query:
+            leaves_strings.append(f"{name} (in {number_issues} issues)")
+
+        pager("\n".join(leaves_strings))
+        print(
+            f"Found {len(leaves)} {kind}s in issues with run_id {self.current_run_id}."
+        )
+
     @catch_keyboard_interrupt()
     @catch_user_error()
     def parents(self) -> None:
@@ -929,8 +971,7 @@ details              show additional information about the current trace frame
     @catch_user_error()
     # pyre-fixme[3]: Return type must be annotated.
     def next_cursor_location(self):
-        """Move cursor to the next trace frame.
-        """
+        """Move cursor to the next trace frame."""
         self._verify_entrypoint_selected()
         self.current_trace_frame_index = min(
             self.current_trace_frame_index + 1, len(self.trace_tuples) - 1
@@ -940,8 +981,7 @@ details              show additional information about the current trace frame
     @catch_user_error()
     # pyre-fixme[3]: Return type must be annotated.
     def prev_cursor_location(self):
-        """Move cursor to the previous trace frame.
-        """
+        """Move cursor to the previous trace frame."""
         self._verify_entrypoint_selected()
         self.current_trace_frame_index = max(self.current_trace_frame_index - 1, 0)
         self.trace()
@@ -1190,8 +1230,7 @@ details              show additional information about the current trace frame
     def _group_trace_frames(
         self, trace_frames: Iterable[TraceFrameQueryResult], limit: int
     ) -> Dict[Tuple[str, str], List[TraceFrameQueryResult]]:
-        """Buckets together trace frames that have the same caller:caller_port.
-        """
+        """Buckets together trace frames that have the same caller:caller_port."""
         caller_buckets: DefaultDict[
             Tuple[str, str], List[TraceFrameQueryResult]
         ] = defaultdict(list)
@@ -1666,8 +1705,7 @@ details              show additional information about the current trace frame
     def _get_callable_from_trace_tuple(
         self, trace_tuple: TraceTuple
     ) -> Tuple[str, str]:
-        """Returns either (caller, caller_port) or (callee, callee_port).
-        """
+        """Returns either (caller, caller_port) or (callee, callee_port)."""
         trace_frame = trace_tuple.trace_frame
         if trace_tuple.placeholder:
             return trace_frame.caller, trace_frame.caller_port
