@@ -11,7 +11,7 @@ import subprocess
 import sys
 from collections import defaultdict
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Tuple, Union, cast
+from typing import Any, Dict, Iterable, List, Optional, Union, cast
 
 import libcst
 import libcst.matchers as libcst_matchers
@@ -23,6 +23,10 @@ LOG: logging.Logger = logging.getLogger(__name__)
 MAX_LINES_PER_FIXME: int = 4
 
 
+PyreError = Dict[str, Any]
+PathsToErrors = Dict[Path, List[PyreError]]
+
+
 class LineBreakTransformer(libcst.CSTTransformer):
     def leave_SimpleWhitespace(
         self,
@@ -192,23 +196,22 @@ class Errors:
 
     def __init__(self, errors: List[Dict[str, Any]]) -> None:
         self.errors: List[Dict[str, Any]] = errors
-        self.error_iterator: Iterator[
-            Tuple[str, Iterator[Dict[str, Any]]]
-        ] = itertools.groupby(sorted(errors, key=error_path), error_path)
-        self.length: int = len(errors)
-
-    def __iter__(self) -> Iterator[Tuple[str, Iterator[Dict[str, Any]]]]:
-        return self.error_iterator.__iter__()
-
-    def __next__(self) -> Tuple[str, Iterator[Dict[str, Any]]]:
-        return self.error_iterator.__next__()
 
     def __len__(self) -> int:
-        return self.length
+        return len(self.errors)
 
     def __eq__(self, other: "Errors") -> bool:
         return self.errors == other.errors
 
+    @property
+    def paths_to_errors(self) -> Dict[str, List[PyreError]]:
+        return {
+            path: list(errors)
+            for path, errors in itertools.groupby(
+                sorted(self.errors, key=error_path), key=error_path
+            )
+        }
+
     def suppress(
         self,
         comment: Optional[str] = None,
@@ -216,9 +219,9 @@ class Errors:
         truncate: bool = False,
         unsafe: bool = False,
     ) -> None:
-        unsuppressed_paths = []
+        unsuppressed_paths_and_exceptions = []
 
-        for path_to_suppress, errors in self:
+        for path_to_suppress, errors in self.paths_to_errors.items():
             LOG.info("Processing `%s`", path_to_suppress)
             try:
                 path = Path(path_to_suppress)
@@ -236,14 +239,18 @@ class Errors:
                 path.write_text(output)
             except SkippingGeneratedFileException:
                 LOG.warning(f"Skipping generated file at {path_to_suppress}")
-            except (ast.UnstableAST, SyntaxError):
-                unsuppressed_paths.append(path_to_suppress)
+            except (ast.UnstableAST, SyntaxError) as exception:
+                unsuppressed_paths_and_exceptions.append((path_to_suppress, exception))
 
-        if unsuppressed_paths:
-            paths_string = ", ".join(unsuppressed_paths)
+        if unsuppressed_paths_and_exceptions:
+            exception_messages = "\n".join(
+                f"{path} - {str(exception)}"
+                for path, exception in unsuppressed_paths_and_exceptions
+            )
             raise PartialErrorSuppression(
-                f"Could not fully suppress errors in: {paths_string}",
-                unsuppressed_paths,
+                "Could not fully suppress errors due to the "
+                f"following exceptions: {exception_messages}",
+                [path for path, _ in unsuppressed_paths_and_exceptions],
             )
 
 
@@ -399,6 +406,42 @@ class SkippingGeneratedFileException(Exception):
     pass
 
 
+def _get_unused_ignore_codes(errors: List[Dict[str, str]]) -> List[int]:
+    unused_ignore_codes: List[int] = []
+    ignore_errors = [error for error in errors if error["code"] == "0"]
+    for error in ignore_errors:
+        match = re.search(
+            r"The `pyre-ignore\[(.*?)\]` or `pyre-fixme\[.*?\]`", error["description"]
+        )
+        if match:
+            unused_ignore_codes.extend(
+                [int(code.strip()) for code in match.group(1).split(",")]
+            )
+    unused_ignore_codes.sort()
+    return unused_ignore_codes
+
+
+def _remove_unused_ignores(line: str, errors: List[Dict[str, str]]) -> str:
+    unused_ignore_codes = _get_unused_ignore_codes(errors)
+    match = re.search(r"pyre-(ignore|fixme)\[([0-9, ]*)\]", line)
+    stripped_line = re.sub(r"# pyre-(ignore|fixme).*$", "", line).rstrip()
+    if not match:
+        return stripped_line
+
+    # One or more codes are specified in the ignore comment.
+    # Remove only the codes that are erroring as unused.
+    ignore_codes_string = match.group(2)
+    ignore_codes = [int(code.strip()) for code in ignore_codes_string.split(",")]
+    remaining_ignore_codes = set(ignore_codes) - set(unused_ignore_codes)
+    if len(remaining_ignore_codes) == 0 or len(unused_ignore_codes) == 0:
+        return stripped_line
+    else:
+        return line.replace(
+            ignore_codes_string,
+            ", ".join([str(code) for code in remaining_ignore_codes]),
+        )
+
+
 def _suppress_errors(
     input: str,
     errors: Dict[int, List[Dict[str, str]]],
@@ -428,8 +471,7 @@ def _suppress_errors(
         number = index + 1
         relevant_errors = errors[number] if number in errors else []
         if any(error["code"] == "0" for error in relevant_errors):
-            # Handle unused ignores.
-            replacement = re.sub(r"# pyre-(ignore|fixme).*$", "", line).rstrip()
+            replacement = _remove_unused_ignores(line, relevant_errors)
             if replacement == "":
                 removing_pyre_comments = True
                 _remove_comment_preamble(new_lines)
@@ -501,7 +543,7 @@ def _error_to_fixme_comment_lines(
 
 
 def _build_error_map(
-    errors: Iterator[Dict[str, Any]]
+    errors: Iterable[Dict[str, Any]]
 ) -> Dict[int, List[Dict[str, str]]]:
     error_map = defaultdict(lambda: [])
     for error in errors:
